/**
 * Created on Aug 21, 2003
 * @author Richard Gradischnegg RG
 */
package isac.bridge;

/**
 * @author rgradisc
 */

import java.io.IOException;
import java.net.ServerSocket;
import java.net.Socket;

import org.apache.log4j.Logger;

/**
 * Clients2KernelServer: runs in its own thread and handles the communication
 * from clients to the kernel
 * 
 * @author Richard Gradischnegg RG
 */
public class Clients2KernelServer implements Runnable {

    private ServerSocket server_ = null;

    private BridgeMain bridge_;

    private static final Logger logger = Logger
            .getLogger(Clients2KernelServer.class.getName());

    /**
     * Constructor: create a new Clients2KernelServer
     * 
     * @param bridge_
     */
    public Clients2KernelServer(BridgeMain bridge) {
        if (logger.isDebugEnabled())
            logger.debug("Clients2KernelServer: bridge_=" + bridge);
        this.bridge_ = bridge;
    }

    /**
     * Listen on socket: Wait for clients to connect to bridge_, and once the
     * connect, start a new ClientOutputWorker that sends the XML-output back to
     * the client
     */
    public void listenSocket() {
        ClientOutputWorker w;
        Socket socket;
        try {
            server_ = new ServerSocket(bridge_.getSocketPort());
        } catch (IOException e) {
            bridge_.log(1, "Could not listen on port "
                    + bridge_.getSocketPort());
            System.out.println("Could not listen on port "
                    + bridge_.getSocketPort());
            System.exit(-1);
        }
        while (true) {
            try {
                // TODO id should be fetched from the BridgeMain:
                // smallestFreeBridgeRmiID
                // (so far: only one BridgeRMI)
                int id = 1;
                socket = server_.accept(); // wait for client connections
                bridge_.log(1, "connection received");
                w = new ClientOutputWorker(id, socket, bridge_);
                Thread t = new Thread(w, "java_bridge_clientworker" + id);
                t.setDaemon(true);
                t.start();
                bridge_.log(1, "new Client accepted, id=" + id);
                // TODO if to much load, start new kernel: e.g.
                // bridge_.startNewKernel();

            } catch (IOException e) {
                bridge_.log(1, "Accept failed on port"
                        + bridge_.getSocketPort());
                System.out.println("Accept failed on port"
                        + bridge_.getSocketPort());
                // Quit program: could not connect to port
                System.exit(-1);
            }
        }
    }

    /**
     * This starts the Client2KernelServer thread
     */
    public void run() {
        listenSocket();
        try {
            bridge_.log(1, "Clients2KernelServer: Cleaning up...");
            server_.close();

        } catch (IOException e) {
            bridge_.log(1, "Could not close socket");
            System.exit(-1);
        }
        bridge_.log(1, "Done cleaning up");
    }
}