/**
 * @author Andreas Griesmayer
 * Created on Oct 6, 2003
 */
package isac.session;

import isac.session.SessionManager;
import isac.browserdialog.IBrowserDialog;
import isac.util.FixedPortRMISocketFactory;
import isac.util.ObjectManagerRMISocketFactory;
import isac.util.ObjectManagerPaths;

import isac.users.UserManager;
import isac.users.IUserManager;

import java.io.IOException;
import java.rmi.Naming;
import java.rmi.RMISecurityManager;
import java.rmi.RemoteException;
import java.rmi.registry.LocateRegistry;
import java.rmi.server.RMISocketFactory;
import java.rmi.server.UnicastRemoteObject;

import org.apache.log4j.Logger;
import org.omg.CORBA.OBJECT_NOT_EXIST;

/**
 * @author Andreas Griesmayer
 * 
 * 
 */
public class ObjectManager extends UnicastRemoteObject implements
		IObjectManager {

	private static final Logger logger = Logger.getLogger(ObjectManager.class.getName());

	static final long serialVersionUID = -7823336348890642600L;

	SessionManager sdialog_ = SessionManager.getInstance();
	
	UserManager user_manager_ = UserManager.getInstance();
	
	WatchdogThread watchdog_;

	/**
	 * fetch the UserManager of the system - multiple calls to this method
	 * result in one and the same UserManager.
	 */
	public IUserManager getUserManager() throws RemoteException {
		if (user_manager_ == null)
			user_manager_ = UserManager.getInstance();
		return user_manager_;
	}

	/**
	 * instantiates the Objectmanager which is used to simulate a function of
	 * dinopolis The ObjectManager provides functions to fetch objects which are
	 * used by other modules of the system using RMI
	 * 
	 * @throws RemoteException
	 */
	protected ObjectManager() throws RemoteException {
		super();
		rmiBind();
		//RK for RMI-problems 07htl: watchdog_ = new WatchdogThread();
		//                           watchdog_.start();
	}

	/**
	 * fetch the XBrowserDialog identified by <code>session</code>
	 */
	public IBrowserDialog getBrowserDialog(String session) throws RemoteException {
		return (IBrowserDialog) sdialog_.getSessionByID(session).getExampleDialog();
	}

	

	/**
	 * fetch the WorksheetDialog identified by <code>wdialog_id</code>
	 */
	/*
	public IWorksheetDialog getWorksheetDialog(String wdialog_id)
			throws RemoteException {
		return (IWorksheetDialog) sdialog_.getWorksheetDialog(wdialog_id);
	}*/

	/**
	 * fetch the XSessionManager of the system - multiple calls to this method
	 * result in one and the same XSessionManager.
	 */
	public ISessionManager getSessionDialog() throws RemoteException {
		if (sdialog_ == null)
			sdialog_ = SessionManager.getInstance();
		return sdialog_;
	}

	/*
	 * instantiates the ObjectManager and registers it on the RMIRegistry on the
	 * local machine as //localhost/isac-ObjectManager . If no RMIRegistry is
	 * running, a new one is started.
	 */
	public static void main(String[] args) {
		try {
			String ini_load_result = ObjectManagerPaths.loadFromFile(args[0]);
			
			if (ini_load_result != null)
				ini_load_result += ObjectManagerPaths.loadFromResource(args[0]);
				
			if (ini_load_result != null)
			{
				System.err.println(ini_load_result);
				System.exit(1);
			}
			
			
	        try
	        {
	                // Use ObjectManagerRMISocketFactory 0704
	                RMISocketFactory.setSocketFactory( new FixedPortRMISocketFactory(ObjectManagerPaths.OBJECT_MANAGER_RMI_PORT));
	                        //new ObjectManagerRMISocketFactory(3113,ObjectManagerPaths.OBJECT_MANAGER_RMI_PORT));
	        } 
	        catch (IOException e)
	        {
	                System.out.println("Exception: " + e.getMessage());
	                e.printStackTrace();
	        }		
	        
	        
			System.out.println(new ObjectManager());

		} catch (RemoteException e) {
			e.printStackTrace();
		}
	}

	private void rmiBind() {
		if (System.getSecurityManager() == null) {
			System.setSecurityManager(new RMISecurityManager());
		}
		
		String name = ObjectManagerPaths.OBJECT_MANAGER_PATH;
		try {
			System.out.println("try to bind as " + name);
			Naming.rebind(name, this);
			System.out.println("Object Manager bound to " + name);
		} catch (java.rmi.ConnectException e) {
			System.err.println("failed to contact as " + name
					+ " (creating RMI-Server on localhost:1099)");
			try {
				LocateRegistry
						.createRegistry(ObjectManagerPaths.OBJECT_MANAGER_PORT);
				rmiBind();
			} catch (java.rmi.RemoteException exc2) {
				System.err.println("can not create registry:"
						+ exc2.getMessage());
			}
		} catch (Exception exc) {
			System.err.println("failed to connect to rmiregistry");
			System.err.println(exc);
		}
	}

}