/*
 * @author Andreas Griesmayer
 * Created on Nov 24, 2003
 */
package isac.util.parser;

import isac.bridge.CMessage;
import isac.util.Formalization;
import isac.util.ResponseWrapper;
import isac.util.Variant;
import isac.util.formulae.CalcHead;
import isac.util.formulae.HierarchyKey;
import isac.util.formulae.ModelItem;
import isac.util.formulae.ModelItemList;
import isac.util.formulae.CalcHeadSimpleID;
import isac.util.formulae.CalcFormula;
import isac.util.formulae.Position;
import isac.util.formulae.Specification;
import isac.util.tactics.*;

import java.io.IOException;
import java.io.StringReader;

import org.apache.commons.digester.Digester;
import org.xml.sax.EntityResolver;
import org.xml.sax.ErrorHandler;
import org.xml.sax.InputSource;
import org.xml.sax.SAXException;
import org.xml.sax.SAXParseException;

/**
 * @author Andreas Griesmayer
 */
public class FormalizationDigest {

    private Digester digester_;

    private String dtd_path_;

    class BridgeErrorHandler implements ErrorHandler {

        public void error(SAXParseException e) {
            System.err.println("---xml-error " + e);
        }

        public void fatalError(SAXParseException e) {
            System.err.println("---xml-fatalerror " + e);
        }

        public void warning(SAXParseException e) {
            System.err.println("---xml-warning " + e);
        }
    }

    // auxillary class for feeding the dtd_
    class BridgeEntityResolver implements EntityResolver {

        String dtd_;

        public BridgeEntityResolver(String dtd) {
            this.dtd_ = dtd;
        }

        public InputSource resolveEntity(String publicId, String systemId)
                throws SAXException, IOException {
            if ((systemId != null) && systemId.endsWith("isac.dtd")) {
                return new InputSource(new StringReader(dtd_));
            } else {
                return null;
            }
        }
    }

    /**
     * Constructor: create a new XMLParserDigest object
     * 
     * @param dtd_path_
     *            Path to the DTD-File (Document Type Description)
     */
    //public FormalizationDigest(String dtd_path_) {
    public FormalizationDigest() {
        digester_ = new Digester();
        //   digester_.setErrorHandler(new BridgeErrorHandler());
        /*
         * this.dtd_path_ = dtd_path_; //dtd_-File --> String (faster
         * validating) StringBuffer sb; try { BufferedReader br = new
         * BufferedReader(new FileReader(dtd_path_)); sb = new StringBuffer();
         * while (br.ready()) { sb.append(br.readLine() + "\n"); } // Set dtd_
         * for parser validation digester_.setEntityResolver(new
         * BridgeEntityResolver(sb.toString())); } catch (FileNotFoundException
         * e) { e.printStackTrace(); } catch (IOException e) {
         * e.printStackTrace(); }
         */

        // Rules for the digester_ to create a formalization
        digester_.addObjectCreate("EXAMPLE/FORMALIZATION", Formalization.class);
        digester_.addObjectCreate("*/FORMALIZATION/VARIANT", Variant.class);
        digester_.addCallMethod("*/TERMLIST/MATHML/ISA", "addNewIsaString", 0);
        digester_.addObjectCreate("*/VARIANT/SPECIFICATION",
                Specification.class);
        digester_.addObjectCreate("*/SPECIFICATION/THEORY",
                CalcHeadSimpleID.class);
        digester_.addCallMethod("*/SPECIFICATION/THEORY", "setID", 0);
        digester_.addSetNext("*/SPECIFICATION/THEORY", "setTheory");

        digester_.addObjectCreate("*/SPECIFICATION/PROBLEM", HierarchyKey.class);
        digester_.addCallMethod("*/SPECIFICATION/PROBLEM/KEY/ID", "addString",
                0);
        digester_.addSetNext("*/SPECIFICATION/PROBLEM", "setProblem");

        digester_.addObjectCreate("*/SPECIFICATION/METHOD", HierarchyKey.class);
        digester_
                .addCallMethod("*/SPECIFICATION/METHOD/KEY/ID", "addString", 0);
        digester_.addSetNext("*/SPECIFICATION/METHOD", "setMethod");

        digester_.addSetNext("*/VARIANT/SPECIFICATION", "setSpecification");
        digester_.addSetNext("*/FORMALIZATION/VARIANT", "addNewVariant");

        // Rules for the digester_ to create Objects and fill them from
        // XML-Files
        digester_.addObjectCreate("ISAC", ResponseWrapper.class);
        digester_.addObjectCreate("*/CALCID", CalcHeadSimpleID.class);
        digester_.addCallMethod("*/CALCID", "setID", 0);
        digester_.addSetRoot("*/CALCID", "setCalcID");

        // Rules for parsing Position
        digester_.addObjectCreate("*/POSITION", Position.class);
        digester_.addCallMethod("*/POSITION/INTLIST/INT", "addInt", 0);
        digester_.addCallMethod("*/POSITION/POS", "setKind", 0);
        digester_.addSetNext("*/POSITION", "setPosition");

        //	Rules for parsing Formula
        digester_.addObjectCreate("*/REFFORMULA/FORMULA", CalcFormula.class);
        digester_
                .addCallMethod("*/REFFORMULA/FORMULA/MATHML/ISA", "setText", 0);
        digester_.addSetNext("*/REFFORMULA/FORMULA", "setResponse");

        //	Rules for parsing CalcHead
        digester_.addObjectCreate("*/CALCHEAD", CalcHead.class);
        digester_.addCallMethod("*/CALCHEAD/HEAD", "setHead", 0);
        digester_.addCallMethod("*/CALCHEAD/BELONGSTO", "setBelongsTo", 0);
        addItemList(digester_, "Given");
        addItemList(digester_, "Where");
        addItemList(digester_, "Find");
        addItemList(digester_, "Relate");
        digester_.addObjectCreate("*/CALCHEAD/SPECIFICATION",
                Specification.class);
        digester_.addObjectCreate("*/CALCHEAD/SPECIFICATION/THEORYID",
                CalcHeadSimpleID.class);
        digester_
                .addCallMethod("*/CALCHEAD/SPECIFICATION/THEORYID", "setID", 0);
        digester_.addSetNext("*/CALCHEAD/SPECIFICATION/THEORYID", "setTheory");
        digester_.addObjectCreate("*/CALCHEAD/SPECIFICATION/PROBLEMID",
                HierarchyKey.class);
        digester_.addCallMethod(
                "*/CALCHEAD/SPECIFICATION/PROBLEMID/STRINGLIST/STRING",
                "addString", 0);
        digester_
                .addSetNext("*/CALCHEAD/SPECIFICATION/PROBLEMID", "setProblem");
        digester_.addObjectCreate("*/CALCHEAD/SPECIFICATION/METHODID",
                HierarchyKey.class);
        digester_.addCallMethod(
                "*/CALCHEAD/SPECIFICATION/METHODID/STRINGLIST/STRING",
                "addString", 0);
        digester_.addSetNext("*/CALCHEAD/SPECIFICATION/METHODID", "setMethod");
        digester_.addSetNext("*/CALCHEAD/SPECIFICATION", "setSpecification");

        digester_.addSetNext("*/CALCHEAD", "setResponse");

        //	Rules for parsing various Tactic Objects
        //	Rules for parsing SimpleTactic
        digester_.addObjectCreate("*/SIMPLETACTIC", SimpleTactic.class);
        digester_.addCallMethod("*/SIMPLETACTIC/MATHML/ISA", "setArgument", 0);
        digester_.addSetProperties("*/SIMPLETACTIC", "name", "name");
        digester_.addSetNext("*/SIMPLETACTIC", "setResponse");

        //	Rules for parsing KeyListTactic
        digester_.addObjectCreate("*/STRINGLISTTACTIC", StringListTactic.class);
        digester_.addCallMethod("*/STRINGLISTTACTIC/STRINGLIST/STRING",
                "addKey", 0);
        digester_.addSetProperties("*/STRINGLISTTACTIC", "name", "name");
        digester_.addSetNext("*/STRINGLISTTACTIC", "setResponse");

        //	Rules for parsing RewriteSetTactic
        digester_.addObjectCreate("*/REWRITESETTACTIC", RewriteSet.class);
        digester_.addCallMethod("*/REWRITESETTACTIC/RULESET", "setRuleSet", 0);
        digester_.addSetProperties("*/REWRITESETTACTIC", "name", "name");
        digester_.addSetNext("*/REWRITESETTACTIC", "setResponse");

        //	Rules for parsing RewriteSetInstTactic
        digester_.addObjectCreate("*/REWRITESETINSTTACTIC",
                RewriteSetInst.class);
        digester_.addCallMethod("*/REWRITESETINSTTACTIC/RULESET", "setRuleSet",
                0);
        digester_.addCallMethod(
                "*/REWRITESETINSTTACTIC/SUBSTITUTION/PAIR/VARIABLE/MATHML/ISA",
                "setVariable", 0);
        digester_.addCallMethod(
                "*/REWRITESETINSTTACTIC/SUBSTITUTION/PAIR/VALUE/MATHML/ISA",
                "setValue", 0);
        digester_.addSetProperties("*/REWRITESETINSTTACTIC", "name", "name");
        digester_.addSetNext("*/REWRITESETINSTTACTIC", "setResponse");

        // Rules for parsing SubProblemTactic
        digester_.addObjectCreate("*/SUBPROBLEMTACTIC", SubProblemTactic.class);
        digester_.addCallMethod("*/SUBPROBLEMTACTIC/THEORY", "setTheoryID", 0);
        digester_.addCallMethod("*/SUBPROBLEMTACTIC/PROBLEM/STRINGLIST/STRING",
                "addProblemString", 0);
        digester_.addSetProperties("*/SUBPROBLEMTACTIC", "name", "name");
        digester_.addSetNext("*/SUBPROBLEMTACTIC", "setResponse");

        //	Rules for parsing CalcIterator move message
        digester_.addObjectCreate("*/CALCITERATOR", CMessage.class);
        digester_.addCallMethod("*/CALCITERATOR/ERROR", "setError", 0);
        digester_.addCallMethod("*/CALCITERATOR/OK", "setOK", 0);
        digester_.addSetNext("*/CALCITERATOR", "setResponse");

        //	Rules for parsing CalcIterator constructor message
        digester_.addObjectCreate("*/ADDUSER", CalcHeadSimpleID.class);
        digester_.addCallMethod("*/ADDUSER/USERID", "setID", 0);
        digester_.addSetNext("*/ADDUSER", "setResponse");
    }

    private void addItemList(Digester digester, String modPart) {
        String modPartUp = modPart.toUpperCase();
        // Rules for parsing ItemLists in the model part of a CalcHead: Given,
        // Find, Where, Relate.
        digester.addObjectCreate("*/CALCHEAD/MODEL/" + modPartUp,
                ModelItemList.class);
        digester.addObjectCreate("*/CALCHEAD/MODEL/" + modPartUp + "/ITEM",
                ModelItem.class);
        digester.addSetProperties("*/CALCHEAD/MODEL/" + modPartUp + "/ITEM",
                "status", "itemStatus");
        digester.addCallMethod("*/CALCHEAD/MODEL/" + modPartUp
                + "/ITEM/MATHML/ISA", "setText", 0);
        digester.addSetNext("*/CALCHEAD/MODEL/" + modPartUp + "/ITEM",
                "addItem");
        digester.addSetNext("*/CALCHEAD/MODEL/" + modPartUp, "set" + modPart);
    }

    /**
     * Set whether or not using the DTD for validating
     * 
     * @param validating
     *            <UL>
     *            <LI>true: XML is tested for validity against DTD
     *            <LI>
     *            <LI>false: XML is only tested for well-formed-ness
     *            <LI>
     *            </UL>
     */
    public void setValidating(boolean validating) {
        digester_.setValidating(validating);
    }

    /**
     * Parse an XML string to Java Object(s)
     * 
     * @param xmlString
     *            the string containing the XML content
     * @return ResponseWrapper containing the CalcID and the parsed Object
     */
    public Formalization parseFormalization(String xmlString) {
        Object o = null;
        //  xmlString =
        //	 "<?xml version=\"1.0\"?>"//<!DOCTYPE ISAC SYSTEM \""
        //+ dtd_path_
        //+ "\">"
        //	   +"<ISAC>"
        //	   xmlString
        //	   + "</ISAC>";
        //;
        xmlString = encodeEntities(xmlString);
        try {
            digester_.clear();
            //System.out.println("root === " + digester_.getRoot());
            o = digester_.parse(new InputSource(new StringReader(xmlString)));
            //System.out.println("--------------->" + xmlString);
        } catch (IOException e) {
            e.printStackTrace();
        } catch (SAXException e) {
            e.printStackTrace();
        }
        Formalization form = (Formalization) o;
        //System.out.println(wrapper.getCalcID().toSMLString());
        return form;
    }

    /**
     * Parse an XML string to Java Object(s)
     * 
     * @param xmlString
     *            the string containing the XML content
     * @return ResponseWrapper containing the CalcID and the parsed Object
     */
    public ResponseWrapper parse(String xmlString) {
        Object o = null;
        xmlString = "<?xml version=\"1.0\"?>"//<!DOCTYPE ISAC SYSTEM \""
                //+ dtd_path_
                //+ "\">"
                //	   +"<ISAC>"
                + xmlString
        //	   + "</ISAC>";
        ;
        encodeEntities(xmlString);
        try {
            digester_.clear();
            //System.out.println("root === " + digester_.getRoot());
            o = digester_.parse(new InputSource(new StringReader(xmlString)));
            //System.out.println("--------------->" + xmlString);
        } catch (IOException e) {
            e.printStackTrace();
        } catch (SAXException e) {
            e.printStackTrace();
        }
        ResponseWrapper wrapper = (ResponseWrapper) o;
        //System.out.println("calcID = "+wrapper.getCalcID());
        return wrapper;
    }

    public static String encodeEntities(String xmlString) {
        xmlString = xmlString.replaceAll(" & ", "&amp;");
        xmlString = xmlString.replaceAll(" > ", "&gt;");
        xmlString = xmlString.replaceAll(" < ", "&lt;");
        
        xmlString = xmlString.replaceAll("&Ouml;", "\u00D6");
        xmlString = xmlString.replaceAll("&ouml;", "\u00F6");
        xmlString = xmlString.replaceAll("&Uuml;", "\u00DC");
        xmlString = xmlString.replaceAll("&uuml;", "\u00FC");
        xmlString = xmlString.replaceAll("&Auml;", "\u00C4");
        xmlString = xmlString.replaceAll("&auml;", "\u00E4");
        xmlString = xmlString.replaceAll("&szlig;","\u00DF");
        return xmlString;
    }

    public static String decodeEntities(String xmlString) {
        xmlString = xmlString.replaceAll("&amp;", " & ");
        xmlString = xmlString.replaceAll("&gt;", " > ");
        xmlString = xmlString.replaceAll("&lt;", " < ");
        return xmlString;
    }
}